# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Models for Invenio-Workflows-Files."""

from invenio_db import db
from invenio_files_rest.models import Bucket
from invenio_workflows.models import WorkflowObjectModel
from sqlalchemy_utils.types import UUIDType


class WorkflowsBuckets(db.Model):
    """Relationship between WorkflowObject and Buckets."""

    __tablename__ = 'workflows_buckets'

    workflow_object_id = db.Column(
        db.Integer,
        db.ForeignKey(
            WorkflowObjectModel.id,
            onupdate='CASCADE',
            ondelete='CASCADE',
        ),
        primary_key=True,
        nullable=False,
    )

    bucket_id = db.Column(
        UUIDType,
        db.ForeignKey(
            Bucket.id,
            onupdate='CASCADE',
            ondelete='CASCADE',
        ),
        primary_key=True,
        nullable=False,
    )

    bucket = db.relationship(Bucket)
    workflow_object = db.relationship(WorkflowObjectModel)

    @classmethod
    def create(cls, workflow_object, bucket):
        """Create a new WorkflowsBuckets and adds it to the session."""
        rb = cls(workflow_object=workflow_object, bucket=bucket)
        db.session.add(rb)
        return rb


__all__ = ('WorkflowsBuckets',)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module adding invenio-files-rest integration workflows."""

from __future__ import absolute_import, print_function


class InvenioWorkflowsFiles(object):
    """Invenio-Workflows-Files extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        app.extensions['invenio-workflows-files'] = self

    def init_config(self, app):
        """Initialize configuration."""
        app.config.setdefault("WORKFLOWS_DEFAULT_STORAGE_CLASS", "S")
        app.config.setdefault("WORKFLOWS_DEFAULT_FILE_LOCATION_NAME", None)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Invenio module adding invenio-files-rest integration to invenio-workflows.
"""

from setuptools import find_packages, setup

README = open('README.rst').read()

TESTS_REQUIRE = [
    'check-manifest>=0.25',
    'coverage>=4.0',
    'isort>=4.2.2',
    'pydocstyle>=1.0.0',
    'pytest-cache>=1.0',
    'pytest-cov>=1.8.0',
    'pytest-pep8>=1.0.6',
    'pytest>=2.8.0',
]

EXTRAS_REQUIRE = {
    'docs': [
        'Sphinx>=1.4.2',
    ],
    'tests': TESTS_REQUIRE,
}

EXTRAS_REQUIRE['all'] = []
for reqs in EXTRAS_REQUIRE.values():
    EXTRAS_REQUIRE['all'].extend(reqs)

SETUP_REQUIRES = [
    'autosemver~=0.2,>=0.2',
    'pytest-runner>=2.6.2',
]

INSTALL_REQUIRES = [
    'autosemver~=0.2,>=0.2',
    'pytest-runner>=2.6.2',
    'invenio-db[versioning]>=1.0.0a9',
    'invenio-files-rest>=1.0.0a3',
    'invenio-records-files>=1.0.0a5',
    'invenio-workflows~=7.0',
]

URL = 'https://github.com/inspirehep/invenio-workflows-files'


setup(
    name='invenio-workflows-files',
    description=__doc__,
    long_description=README,
    keywords='invenio TODO',
    license='GPLv2',
    author='CERN',
    author_email='admin@inspirehep.net',
    url=URL,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'invenio_base.api_apps': [
            'invenio_workflows_files '
            '= invenio_workflows_files:InvenioWorkflowsFiles',
        ],
        'invenio_base.apps': [
            'invenio_workflows_files '
            '= invenio_workflows_files:InvenioWorkflowsFiles',
        ],
        'invenio_db.alembic': [
            'invenio_workflows_files = invenio_workflows_files:alembic',
        ],
        'invenio_i18n.translations': [
            'messages = invenio_workflows_files',
        ],
        'invenio_db.models': [
            'invenio_workflows_files = invenio_workflows_files.models'
        ],
    },
    extras_require=EXTRAS_REQUIRE,
    install_requires=INSTALL_REQUIRES,
    setup_requires=SETUP_REQUIRES,
    tests_require=TESTS_REQUIRE,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Development Status :: 1 - Planning',
    ],
    autosemver={
        'bugtracker_url': URL + '/issues/',
    }
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Alembic migration 3255c6f9aca0."""
import sqlalchemy_utils
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.

revision = '3255c6f9aca0'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    """Upgrades alembic."""
    op.create_table('workflows_buckets',
                    sa.Column('workflow_object_id', sa.Integer(),
                              nullable=False),
                    sa.Column('bucket_id',
                              sqlalchemy_utils.types.uuid.UUIDType(length=16),
                              nullable=False),
                    sa.ForeignKeyConstraint(['bucket_id'],
                                            [u'files_bucket.id'],
                                            name=op.f(
                                                'fk_workflows_buckets_bucket'
                                                '_id_files_bucket'),
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                    sa.ForeignKeyConstraint(['workflow_object_id'],
                                            [u'workflows_object.id'],
                                            name=op.f(
                                                'fk_workflows_buckets_workflow'
                                                '_object_id_workflows_object'),
                                            onupdate='CASCADE',
                                            ondelete='CASCADE'),
                    sa.PrimaryKeyConstraint('workflow_object_id', 'bucket_id',
                                            name=op.f('pk_workflows_buckets'))
                    )


def downgrade():
    """Downgrades alembic."""
    op.drop_table('workflows_buckets')



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.

"""Version information for Invenio-Workflows-Files."""

from collections import OrderedDict

from flask import current_app
from invenio_files_rest.models import Bucket
from invenio_records_files.api import FileObject, FilesIterator
from invenio_workflows import WorkflowObject as _WorkflowObject

from .models import WorkflowsBuckets


class WorkflowFilesIterator(FilesIterator):
    """Iterator for files."""

    def __init__(self, workflow_object, bucket, file_cls=None):
        """Initialize iterator."""
        self._it = None
        self.record = workflow_object.data
        self.model = workflow_object
        self.file_cls = file_cls or FileObject
        self.bucket = bucket
        self.record.setdefault('_files', [])
        self.filesmap = OrderedDict([
            (f['key'], f) for f in self.record['_files']
        ])


class WorkflowObject(_WorkflowObject):
    """Define API for files manipulation using ``FilesMixin``."""

    files_iterator = WorkflowFilesIterator
    file_cls = FileObject

    def _create_bucket(self, location=None, storage_class=None):
        """Create file bucket for workflow object."""
        if location is None:
            location = current_app.config[
                'WORKFLOWS_DEFAULT_FILE_LOCATION_NAME'
            ]
        if storage_class is None:
            storage_class = current_app.config[
                'WORKFLOWS_DEFAULT_STORAGE_CLASS'
            ]
        bucket = Bucket.create(
            location=location,
            storage_class=storage_class
        )
        WorkflowsBuckets.create(workflow_object=self.model, bucket=bucket)
        return bucket

    @property
    def files(self):
        """Get files iterator."""
        workflows_bucket = WorkflowsBuckets.query.filter_by(
            workflow_object_id=self.id).first()

        if not workflows_bucket:
            bucket = self._create_bucket()
        else:
            bucket = workflows_bucket.bucket

        return self.files_iterator(self, bucket=bucket, file_cls=self.file_cls)
